#ifndef MREQ_H
#define MREQ_H

#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <netdb.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <netinet/in.h>
#include <arpa/inet.h>

class Mreq {

	protected:
		int family;

	public:
		virtual int join(int sockfd) = 0;
		virtual int drop(int sockfd) = 0;
		virtual void setGroup(char *group, char *sources[], int numsrc, int fmode) = 0;
		virtual char *getGroup() = 0;
		int getFamily() { return family; }
		static Mreq *getMClass(char *address, int numsrc);
		static int getAddrType(char *address);

};

class MreqV4 : public Mreq {

	private:
		struct ip_mreq mreq;

	public:
		MreqV4() : Mreq() { family = AF_INET; }
		int join(int sockfd);
		int drop(int sockfd);
		void setGroup(char *group, char *sources[], int numsrc, int fmode);
		char *getGroup();

};

#ifdef IPV6_ADD_MEMBERSHIP

class MreqV6 : public Mreq {

	private:
		struct ipv6_mreq mreq;

	public:
		MreqV6() : Mreq() { family = AF_INET6; }
		int join(int sockfd);
		int drop(int sockfd);
		void setGroup(char *group, char *sources[], int numsrc, int fmode);
		char *getGroup();

};

#endif

#ifdef SIOCSIPMSFILTER

class MreqV4Filter : public Mreq {

	private:
		struct ip_msfilter *mreq;

	public:
		MreqV4Filter() : Mreq() { family = AF_INET; }
		int join(int sockfd);
		int drop(int sockfd);
		void setGroup(char *group, char *sources[], int numsrc, int fmode);
		char *getGroup();

};

#endif

#endif
